﻿--[[
	MrtWoo_StatusBar: Shows useful information on the status bar
	Copyright (C) 2010 Pavel Dudkovsky (mrtime@era.by)
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
]]

local MrtWoo = MrtWoo;
local StatusBar = MrtWoo:NewModule("StatusBar");
StatusBar.Version = "v0.1(api 4.0.0)"

local opts = {
	sb_options = {
		type="group",
		name="StatusBar",
		args={

			exp = {
				type = "toggle",
				name = "Показывать опыт?",
				get = function() return MrtWoo.db.profile.modules.sb_exp end,
				set = function() MrtWoo.db.profile.modules.sb_exp = not MrtWoo.db.profile.modules.sb_exp end,
				order=1,
			},
				
			short = {
				type = "toggle",
				name = "Опыт в коротком виде (1k..)?",
				get = function() return MrtWoo.db.profile.modules.sb_exp_short end,
				set = function() MrtWoo.db.profile.modules.sb_exp_short = not MrtWoo.db.profile.modules.sb_exp_short end,
				order=2,
			},
			
			maxlvl = {
				type = "range",
				max = 80,
				min = 1,
				bigStep = 1,
				step = 1,
				name = "Максимальный уровень получения опыыта",
				get = function() return MrtWoo.db.profile.modules.sb_exp_maxlvl end,
				set = function(info, input) MrtWoo.db.profile.modules.sb_exp_maxlvl = input end,
				order=3,
			},
			
			rep = {
				type = "toggle",
				name = "Показывать репутацию?",
				get = function() return MrtWoo.db.profile.modules.sb_rep end,
				set = function() MrtWoo.db.profile.modules.sb_rep = not MrtWoo.db.profile.modules.sb_rep end,
				order=4,
			},
						
			emblems = {
				type = "toggle",
				name = "Показывать эмблемы?",
				get = function() return MrtWoo.db.profile.modules.sb_emblems end,
				set = function() MrtWoo.db.profile.modules.sb_emblems = not MrtWoo.db.profile.modules.sb_emblems end,
				order=5,
			},
			
			align = {
				values = {"Сверху", "Снизу"},
				type = "select",
				name = "Позиция",
				get = function() return MrtWoo.db.profile.modules.sb_align end,
				set = function(info, input) MrtWoo.db.profile.modules.sb_align = input end,
				order=5,
			}				
				
		},
	}
};

StatusBar.FACTION_COLORS = {
	[0] = "FFFFFF",
	[1] = "CC4C38",
	[2] = "CC4C38",
	[3] = "BF4300",
	[4] = "E5B200",
	[5] = "009919",
	[6] = "009919",
	[7] = "009919",
	[8] = "009919",
};

StatusBar.UpdateLast = 0; -- sec
StatusBar.UpdateInterval = 2; -- sec

function StatusBar:SetDefaults()
	MrtWoo.defaults.profile.modules.sb_exp = true;
	MrtWoo.defaults.profile.modules.sb_exp_short = true;
	MrtWoo.defaults.profile.modules.sb_exp_maxlvl = 80;
	MrtWoo.defaults.profile.modules.sb_rep = true;
	MrtWoo.defaults.profile.modules.sb_emblems = true;
	MrtWoo.defaults.profile.modules.sb_align = "BOTTOM";
end;

function StatusBar:OnInitialize()
			
	self.SetDefaults();
	
	self.Wnd = CreateFrame("Frame", "MrtWooStatusBar", UIParent);
	self.Wnd:Hide();
	self.Wnd:SetPoint("CENTER", UIParent, "CENTER", 0, 0);
	
	if MrtWoo.db.profile.modules.sb_align == 2 then	
		self.Wnd:SetPoint("BOTTOM", UIParent, "BOTTOM", 0, 0);
	else
		self.Wnd:SetPoint("TOP", UIParent, "TOP", 0, 0);
	end;
		
	self.Wnd:SetHeight(20);
	self.Wnd:SetWidth(2000);
	
	--Create Font String
	self.Font = self.Wnd:CreateFontString("MrtWooStatusBarFont", "OVERLAY");
	self.Font:SetFontObject(GameFontNormal);
	self.Font:SetHeight(20);
	self.Font:SetAllPoints(self.Wnd);

	--Create Background
	self.Texture = self.Wnd:CreateTexture("MrtWooStatusBarTexture", "BACKGROUND");
	self.Texture:SetHeight(20);
	self.Texture:SetWidth(2000);
	self.Texture:SetAllPoints(self.Wnd);
	self.Texture:SetTexture(0, 0, 0, 0.5);
	
	self.Wnd:SetScript("OnUpdate", StatusBar.OnUpdate);
	
	table.insert(MrtWoo.options.plugins, opts)
end;

function StatusBar:GetEmblemsInfo()
	local name, count, icon, header, expanded;
	local info = "";

	for i=1,GetCurrencyListSize(),1 do
		name,header,expanded,_,watched,count,_,icon,_ = GetCurrencyListInfo(i);
		if not header and watched then
			info = info .. ", " .. name..": " .. count;
		end;
	end;

	return string.sub(info, 2);
end;

function StatusBar:IsShowExp()
	return MrtWoo.db.profile.modules.sb_exp;
end;

function StatusBar:IsShowReputation()
	return MrtWoo.db.profile.modules.sb_rep;
end;

function StatusBar:IsShowEmblems()
	return MrtWoo.db.profile.modules.sb_emblems;
end;

function StatusBar:IsExpShort()
	return MrtWoo.db.profile.modules.sb_exp_short;
end;

function StatusBar:UpdateLevel()
	StatusBar.level = UnitLevel("player");
end;

function StatusBar:IsExpLevel()
	return MrtWoo.db.profile.modules.sb_exp_maxlvl < StatusBar.level;
end;

function StatusBar:GetExperience()
	local self = StatusBar;
	local experience = "";

	self.UpdateLevel();
		 
	if self.IsShowExp() and self.IsExpLevel() then
		local xp = UnitXP("player");
		local mx = UnitXPMax("player");
		local color = "|cffFF00FF ";
		local ender = "|r";
		local xps = xp;
		local mxs = mx;
		
		if self.IsExpShort() then
			if xp >= 1000 then
				xps = math.floor(xp/1000).."k";
			end
			
			if mx >= 1000 then
				mxs = math.floor(mx/1000).."k";
			end
		end

		local rested = GetXPExhaustion();
		if rested then
			rested = rested / 2;
			if self.IsExpShort() then
				if rested >= 1000 then
					rested = math.floor(rested / 1000).."k";
				end
			end

			rested = " (+"..rested..")";

			color = "|cff0080FF";
		else
			rested = "";
		end			

		local rexp = xps..rested.." / "..mxs;

		local percent  = " ("..math.floor((xp/mx)*100).."%) ";			

		experience = color .. "lvl: ".. StatusBar.level .. ", ".. rexp .. percent .. ender ;
	end;
	
	return experience;
	
end;

function StatusBar:GetReputation()
	local self = StatusBar;
	local reputation = "";
	
	if self.IsShowReputation() then
		local name, standing, minrep, maxrep, value = GetWatchedFactionInfo()
		if name then
			local color = "|cff"..self.FACTION_COLORS[standing];
			local ender = " |r";
			value = value - minrep;
			local limit = maxrep - minrep;
			local fraction = " "..value.." / "..limit;
			local percent  = " ("..math.floor((value/limit)*100).."%) ";
			local stand = "";
			
			if standing == 0 then
				stand = "";
			elseif standing == 1 then
				stand = "(Ненависть)";
			elseif standing == 2 then
				stand = "(Враждебность)";
			elseif standing == 3 then
				stand = "(Неприязнь)";
			elseif standing == 4 then
				stand = "(Равнодушие)";
			elseif standing == 5 then
				stand = "(Дружелюбие)";
			elseif standing == 6 then
				stand = "(Уважение)";
			elseif standing == 7 then
				stand = "(Почтение)";
			elseif standing == 8 then
				stand = "(Превознесение)";
			end
			
			reputation = color.."\""..name.."\""..fraction..percent..stand..ender;
		end;
	end;		
	return reputation;
end;

function StatusBar:OnUpdate(elapsed)
	-- MrtWoo:Print(elapsed);
	local self = StatusBar;
	self.UpdateLast = self.UpdateLast + elapsed;
	
	if (self.UpdateLast > self.UpdateInterval) then
		
		local exp = self.GetExperience();
		local rep = self.GetReputation();
		
		local txtresult = exp..rep;
		
		if self.IsShowEmblems() then
			local emblems = self.GetEmblemsInfo();
			if (string.len(txtresult) > 0) and (string.len(emblems ) > 0) then
				emblems = ", "..emblems;
			end;
			txtresult = txtresult..emblems;
		end;
		
		if (string.len(txtresult) <= 0) then
			txtresult = "MrtWoo StatusBar " .. self.Version;
		end;
				
		self.Font:SetText(txtresult);
		
		StatusBar.UpdateLast = 0;
	end;
end;

function StatusBar:OnEnable()
	self.Wnd:Show();
end;

function StatusBar:OnDisable()
	self.Wnd:Hide();
end;
